#!/usr/bin/env python

class Singleton(object):

    __instance = None

    def __new__(cls, *args, **kwargs):
        # Allow subclasses to create their own instances.
        if cls.__instance is None or cls != type(cls.__instance):
            instance = object.__new__(cls, *args, **kwargs)
            cls.__instance = instance
            instance.__init__(*args, **kwargs)
        return cls.__instance

    @classmethod
    def getInstance(cls, *args, **kwargs):
        return cls.__new__(cls, *args, **kwargs)

    def __init__(self, *args, **kwargs):
        # Only initialize once.
        if Singleton.__instance is not None:
            return

####################################################################
# Testing
if __name__ == '__main__':
    s1 = Singleton.getInstance()
    s2 = Singleton()
    s3 = Singleton.getInstance()
    s4 = Singleton()
    print 'S1: ', s1
    print 'S2: ', s2
    print 'S3: ', s3
    print 'S4: ', s4
    class A(Singleton):
        def __init__(self, x):
            self.x = x
    class B(Singleton):
        def __init__(self, x):
            self.x = x
    class C(Singleton):
        def __init__(self):
            self.x = 14

    a1 = A(3)
    a2 = A(4)
    print a1
    print a2
    print a2.x, a1.x

    b1 = B(6)
    b2 = B(5)
    print b1
    print b2
    print b2.x, b1.x

    c1 = C()
    c2 = C()
    print c1
    print c2
    print c2.x, c1.x
